rm(list=ls())

# load packages
library(deSolve)

# get data
# dispersal
dispersal_posteriors <- read.table(file="dispRates_posteriors.txt", header=T)

# load posteriors
popGrowth_posteriors <- read.table(file="logGrowth_clones_posteriors.txt", header=T)

# summarize all
lines_data <- read.table(file="dispersal_estimates.txt", header=T)
# disperal is per three hours
lines_data$disp <- NA

# data structure for storage
all_winners_front <- numeric()
all_winners_core <- numeric()
all_winners_remix <- numeric()

# no of iterations
no_glob_it <- 10000

##################################################
##################################################
# iterate from here
for(act_glob_it in 1:no_glob_it){
  
  lines_data$r0 <- NA
  lines_data$alpha <- NA
  
  #################################
  # selection of current r0 and alpha values
  for(i in 1:length(lines_data$clone)){
    
    act_val <- sample(which(popGrowth_posteriors$clone_names==lines_data$clone[i]),1)
    
    lines_data$r0[i] <- popGrowth_posteriors$r0[act_val]
    lines_data$alpha[i] <- popGrowth_posteriors$alpha[act_val]
    
    lines_data$disp[i] <- sample(dispersal_posteriors[,lines_data$clone[i]],1)
  }
  
  parameter.names <- numeric()
  for (i in 1:20){ parameter.names[i] <- paste("r0_",i,sep="") }
  for (i in 21:40){ parameter.names[i] <- paste("alpha_",i-20,sep="") }
  for (i in 41:60){ parameter.names[i] <- paste("disp_",i-40,sep="") }
  
  parameters <- c(lines_data$r0, lines_data$alpha, lines_data$disp)
  names(parameters) <- parameter.names
  
  names.start_pop_sizes_begin <- numeric()
  for(i in 1:20){names.start_pop_sizes_begin[i] <- paste("N_",i,sep="")}
  
  start_pop_sizes_begin <- rep(10,20)
  names(start_pop_sizes_begin) <- names.start_pop_sizes_begin
  
  # parameters
  max_it <- 10
  growth_time <- 24*6
  disp_time <- 3
  
  quasi_extinction_threshold <- 0.7
  
  ################################################
  # define growth model
  # log growth function
  LogGrowth <- function(time,y,parms){
    with(as.list(c(y,parms)), {
      
      all_comp <- alpha_1*N_1 + alpha_2*N_2 + alpha_3*N_3 + alpha_4*N_4 + alpha_5*N_5 + alpha_6*N_6 + alpha_7*N_7 + alpha_8*N_8 + alpha_9*N_9 + alpha_10*N_10 + alpha_11*N_11 + alpha_12*N_12 + alpha_13*N_13 + alpha_14*N_14 + alpha_15*N_15 + alpha_16*N_16 + alpha_17*N_17 + alpha_18*N_18 + alpha_19*N_19 + alpha_20*N_20
      
      dN_1 <- (r0_1 - all_comp ) * N_1
      dN_2 <- (r0_2 - all_comp) * N_2
      dN_3 <- (r0_3 - all_comp) * N_3
      dN_4 <- (r0_4 - all_comp) * N_4
      dN_5 <- (r0_5 - all_comp) * N_5
      dN_6 <- (r0_6 - all_comp) * N_6
      dN_7 <- (r0_7 - all_comp) * N_7
      dN_8 <- (r0_8 - all_comp) * N_8
      dN_9 <- (r0_9 - all_comp) * N_9
      dN_10 <- (r0_10 - all_comp) * N_10
      dN_11 <- (r0_11 - all_comp) * N_11
      dN_12 <- (r0_12 - all_comp) * N_12
      dN_13 <- (r0_13 - all_comp) * N_13
      dN_14 <- (r0_14 - all_comp) * N_14
      dN_15 <- (r0_15 - all_comp) * N_15
      dN_16 <- (r0_16 - all_comp) * N_16
      dN_17 <- (r0_17 - all_comp) * N_17
      dN_18 <- (r0_18 - all_comp) * N_18
      dN_19 <- (r0_19 - all_comp) * N_19
      dN_20 <- (r0_20 - all_comp) * N_20
      
      list(c(dN_1,dN_2,dN_3,dN_4,dN_5,dN_6,dN_7,dN_8,dN_9,dN_10,dN_11,dN_12,dN_13,dN_14,dN_15,dN_16,dN_17,dN_18,dN_19,dN_20))
    })
  }
  
  # log growth function with dispersal
  LogGrowth_and_dispersal <- function(time,y,parms){
    with(as.list(c(y,parms)), {
      
      all_comp_p1 <- alpha_1*N_1_p1 + alpha_2*N_2_p1 + alpha_3*N_3_p1 + alpha_4*N_4_p1 + alpha_5*N_5_p1 + alpha_6*N_6_p1 + alpha_7*N_7_p1 + alpha_8*N_8_p1 + alpha_9*N_9_p1 + alpha_10*N_10_p1 + alpha_11*N_11_p1 + alpha_12*N_12_p1 + alpha_13*N_13_p1 + alpha_14*N_14_p1 + alpha_15*N_15_p1 + alpha_16*N_16_p1 + alpha_17*N_17_p1 + alpha_18*N_18_p1 + alpha_19*N_19_p1 + alpha_20*N_20_p1
      all_comp_p2 <- alpha_1*N_1_p2 + alpha_2*N_2_p2 + alpha_3*N_3_p2 + alpha_4*N_4_p2 + alpha_5*N_5_p2 + alpha_6*N_6_p2 + alpha_7*N_7_p2 + alpha_8*N_8_p2 + alpha_9*N_9_p2 + alpha_10*N_10_p2 + alpha_11*N_11_p2 + alpha_12*N_12_p2 + alpha_13*N_13_p2 + alpha_14*N_14_p2 + alpha_15*N_15_p2 + alpha_16*N_16_p2 + alpha_17*N_17_p2 + alpha_18*N_18_p2 + alpha_19*N_19_p2 + alpha_20*N_20_p2
      
      dN_1_p1 <- (r0_1 - all_comp_p1) * N_1_p1 + N_1_p2 * disp_1 - N_1_p1 * disp_1
      dN_2_p1 <- (r0_2 - all_comp_p1) * N_2_p1 + N_2_p2 * disp_2 - N_2_p1 * disp_2
      dN_3_p1 <- (r0_3 - all_comp_p1) * N_3_p1 + N_3_p2 * disp_3 - N_3_p1 * disp_3
      dN_4_p1 <- (r0_4 - all_comp_p1) * N_4_p1 + N_4_p2 * disp_4 - N_4_p1 * disp_4
      dN_5_p1 <- (r0_5 - all_comp_p1) * N_5_p1 + N_5_p2 * disp_5 - N_5_p1 * disp_5
      dN_6_p1 <- (r0_6 - all_comp_p1) * N_6_p1 + N_6_p2 * disp_6 - N_6_p1 * disp_6
      dN_7_p1 <- (r0_7 - all_comp_p1) * N_7_p1 + N_7_p2 * disp_7 - N_7_p1 * disp_7
      dN_8_p1 <- (r0_8 - all_comp_p1) * N_8_p1 + N_8_p2 * disp_8 - N_8_p1 * disp_8
      dN_9_p1 <- (r0_9 - all_comp_p1) * N_9_p1 + N_9_p2 * disp_9 - N_9_p1 * disp_9
      dN_10_p1 <- (r0_10 - all_comp_p1) * N_10_p1 + N_10_p2 * disp_10 - N_10_p1 * disp_10
      dN_11_p1 <- (r0_11 - all_comp_p1) * N_11_p1 + N_11_p2 * disp_11 - N_11_p1 * disp_11
      dN_12_p1 <- (r0_12 - all_comp_p1) * N_12_p1 + N_12_p2 * disp_12 - N_12_p1 * disp_12
      dN_13_p1 <- (r0_13 - all_comp_p1) * N_13_p1 + N_13_p2 * disp_13 - N_13_p1 * disp_13
      dN_14_p1 <- (r0_14 - all_comp_p1) * N_14_p1 + N_14_p2 * disp_14 - N_14_p1 * disp_14
      dN_15_p1 <- (r0_15 - all_comp_p1) * N_15_p1 + N_15_p2 * disp_15 - N_15_p1 * disp_15
      dN_16_p1 <- (r0_16 - all_comp_p1) * N_16_p1 + N_16_p2 * disp_16 - N_16_p1 * disp_16
      dN_17_p1 <- (r0_17 - all_comp_p1) * N_17_p1 + N_17_p2 * disp_17 - N_17_p1 * disp_17
      dN_18_p1 <- (r0_18 - all_comp_p1) * N_18_p1 + N_18_p2 * disp_18 - N_18_p1 * disp_18
      dN_19_p1 <- (r0_19 - all_comp_p1) * N_19_p1 + N_19_p2 * disp_19 - N_19_p1 * disp_19
      dN_20_p1 <- (r0_20 - all_comp_p1) * N_20_p1 + N_20_p2 * disp_20 - N_20_p1 * disp_20
      
      dN_1_p2 <- (r0_1 - all_comp_p2) * N_1_p2 + N_1_p1 * disp_1 - N_1_p2 * disp_1
      dN_2_p2 <- (r0_2 - all_comp_p2) * N_2_p2 + N_2_p1 * disp_2 - N_2_p2 * disp_2
      dN_3_p2 <- (r0_3 - all_comp_p2) * N_3_p2 + N_3_p1 * disp_3 - N_3_p2 * disp_3
      dN_4_p2 <- (r0_4 - all_comp_p2) * N_4_p2 + N_4_p1 * disp_4 - N_4_p2 * disp_4
      dN_5_p2 <- (r0_5 - all_comp_p2) * N_5_p2 + N_5_p1 * disp_5 - N_5_p2 * disp_5
      dN_6_p2 <- (r0_6 - all_comp_p2) * N_6_p2 + N_6_p1 * disp_6 - N_6_p2 * disp_6
      dN_7_p2 <- (r0_7 - all_comp_p2) * N_7_p2 + N_7_p1 * disp_7 - N_7_p2 * disp_7
      dN_8_p2 <- (r0_8 - all_comp_p2) * N_8_p2 + N_8_p1 * disp_8 - N_8_p2 * disp_8
      dN_9_p2 <- (r0_9 - all_comp_p2) * N_9_p2 + N_9_p1 * disp_9 - N_9_p2 * disp_9
      dN_10_p2 <- (r0_10 - all_comp_p2) * N_10_p2 + N_10_p1 * disp_10 - N_10_p2 * disp_10
      dN_11_p2 <- (r0_11 - all_comp_p2) * N_11_p2 + N_11_p1 * disp_11 - N_11_p2 * disp_11
      dN_12_p2 <- (r0_12 - all_comp_p2) * N_12_p2 + N_12_p1 * disp_12 - N_12_p2 * disp_12
      dN_13_p2 <- (r0_13 - all_comp_p2) * N_13_p2 + N_13_p1 * disp_13 - N_13_p2 * disp_13
      dN_14_p2 <- (r0_14 - all_comp_p2) * N_14_p2 + N_14_p1 * disp_14 - N_14_p2 * disp_14
      dN_15_p2 <- (r0_15 - all_comp_p2) * N_15_p2 + N_15_p1 * disp_15 - N_15_p2 * disp_15
      dN_16_p2 <- (r0_16 - all_comp_p2) * N_16_p2 + N_16_p1 * disp_16 - N_16_p2 * disp_16
      dN_17_p2 <- (r0_17 - all_comp_p2) * N_17_p2 + N_17_p1 * disp_17 - N_17_p2 * disp_17
      dN_18_p2 <- (r0_18 - all_comp_p2) * N_18_p2 + N_18_p1 * disp_18 - N_18_p2 * disp_18
      dN_19_p2 <- (r0_19 - all_comp_p2) * N_19_p2 + N_19_p1 * disp_19 - N_19_p2 * disp_19
      dN_20_p2 <- (r0_20 - all_comp_p2) * N_20_p2 + N_20_p1 * disp_20 - N_20_p2 * disp_20
      
      list(c(dN_1_p1, dN_2_p1, dN_3_p1, dN_4_p1, dN_5_p1, dN_6_p1, dN_7_p1, dN_8_p1, dN_9_p1, dN_10_p1, dN_11_p1, dN_12_p1, dN_13_p1, dN_14_p1, dN_15_p1, dN_16_p1, dN_17_p1, dN_18_p1, dN_19_p1, dN_20_p1, dN_1_p2, dN_2_p2, dN_3_p2, dN_4_p2, dN_5_p2, dN_6_p2, dN_7_p2, dN_8_p2, dN_9_p2, dN_10_p2, dN_11_p2, dN_12_p2, dN_13_p2, dN_14_p2, dN_15_p2, dN_16_p2, dN_17_p2, dN_18_p2, dN_19_p2, dN_20_p2))
    })
  }
  
  ###################################################
  # data structure
  N_front <- matrix(ncol=max_it,nrow=20)
  N_core <- matrix(ncol=max_it,nrow=20)
  N_remix <- matrix(ncol=max_it,nrow=20)
  
  ##################################################
  # run model
  
  # initialize pop sizes
  start_pop_sizes_front <- start_pop_sizes_begin
  start_pop_sizes_core <- start_pop_sizes_begin
  start_pop_sizes_remix <- start_pop_sizes_begin
  
  help_names <- numeric()
  help_names2 <- numeric()
  for(hc in 1:20){
    help_names[hc] <- paste("N_",hc,"_p1",sep="")
    help_names2[hc] <-paste("N_",hc,sep="")
  }
  for(hc in 21:40){help_names[hc] <- paste("N_",hc-20,"_p2",sep="")}
  
  # begin to iterate
  for (act_it in 1:max_it){
    
    # first growth
    growth_front <- ode(y=start_pop_sizes_front, times=c(0,growth_time), LogGrowth, parameters)
    growth_core <- ode(y=start_pop_sizes_core, times=c(0,growth_time), LogGrowth, parameters)
    growth_remix <- ode(y=start_pop_sizes_remix, times=c(0,growth_time), LogGrowth, parameters)
    
    # take the pop sizes after growth to start dispersal
    # for front treatment
    if (act_it == 1){
      pop_sizes_after_growth_front <- c(start_pop_sizes_front,rep(0,20))
    }else{
      pop_sizes_after_growth_front <- c(growth_front[2,2:21],rep(0,20))
      pop_sizes_after_growth_front <- pop_sizes_after_growth_front/sum(pop_sizes_after_growth_front)*80
    }
    
    pop_sizes_after_growth_front[which(is.na(pop_sizes_after_growth_front))] <- 0
    
    pop_sizes_after_growth_front[which(pop_sizes_after_growth_front < quasi_extinction_threshold)] <- 0
    names(pop_sizes_after_growth_front) <- help_names
    
    # for core treatment
    if(act_it == 1){
      pop_sizes_after_growth_core <- c(start_pop_sizes_core,rep(0,20))
    }else{
      pop_sizes_after_growth_core <- c(growth_core[2,2:21],rep(0,20))
      pop_sizes_after_growth_core <- pop_sizes_after_growth_core/sum(pop_sizes_after_growth_core)*80
    }
    
    pop_sizes_after_growth_core[which(is.na(pop_sizes_after_growth_core))] <- 0
    
    pop_sizes_after_growth_core[which(pop_sizes_after_growth_core < quasi_extinction_threshold)] <- 0
    names(pop_sizes_after_growth_core) <- help_names
    
    # for remix treatment
    if(act_it == 1){
      pop_sizes_after_growth_remix <- c(start_pop_sizes_remix,rep(0,20))
    }else{
      pop_sizes_after_growth_remix <- c(growth_remix[2,2:21],rep(0,20)) 
      pop_sizes_after_growth_remix <- pop_sizes_after_growth_remix/sum(pop_sizes_after_growth_remix)*80
    }
    
    pop_sizes_after_growth_remix[which(is.na(pop_sizes_after_growth_remix))] <- 0
    
    
    pop_sizes_after_growth_remix[which(pop_sizes_after_growth_remix < quasi_extinction_threshold)] <- 0
    names(pop_sizes_after_growth_remix) <- help_names
    
    
    for (act_clone in 1:20){
      if(act_it==1){
        N_front[act_clone,act_it] <- as.numeric(start_pop_sizes_front[act_clone])
        N_core[act_clone,act_it] <- as.numeric(start_pop_sizes_core[act_clone])  
        N_remix[act_clone,act_it] <- as.numeric(start_pop_sizes_remix[act_clone])  
      }else{
        N_front[act_clone,act_it] <- as.numeric(growth_front[2,act_clone+1])
        N_core[act_clone,act_it] <- as.numeric(growth_core[2,act_clone+1])  
        N_remix[act_clone,act_it] <- as.numeric(growth_remix[2,act_clone+1])  
      }
    }
    
    # then dispersal and growth
    growth_and_dispersal_front <- ode(y=pop_sizes_after_growth_front, times=c(0,disp_time), LogGrowth_and_dispersal, parameters)
    growth_and_dispersal_core <- ode(y=pop_sizes_after_growth_core, times=c(0,disp_time), LogGrowth_and_dispersal, parameters)
    growth_and_dispersal_remix <- ode(y=pop_sizes_after_growth_remix, times=c(0,disp_time), LogGrowth_and_dispersal, parameters)
    
    # take the densities in patch 2 to start new round
    # for front treatment
    start_pop_sizes_front <- growth_and_dispersal_front[2,22:41] 
    names(start_pop_sizes_front) <- help_names2
    #only take 10 per mL as in the experiment
    start_pop_sizes_front <- start_pop_sizes_front /sum(start_pop_sizes_front)*10
    
    start_pop_sizes_front[which(is.na(start_pop_sizes_front))] <- 0
    
    start_pop_sizes_front[which(start_pop_sizes_front < quasi_extinction_threshold)] <- 0
    
    # for core treatment
    start_pop_sizes_core <- growth_and_dispersal_core[2,2:21]
    names(start_pop_sizes_core) <- help_names2
    #only take 10 per mL as in the experiment
    start_pop_sizes_core <- start_pop_sizes_core /sum(start_pop_sizes_core)*10
    
    start_pop_sizes_core[which(is.na(start_pop_sizes_core))] <- 0
    
    start_pop_sizes_core[which(start_pop_sizes_core < quasi_extinction_threshold)] <- 0
    
    # for remix treatment
    start_pop_sizes_remix <- growth_and_dispersal_remix[2,2:21] + growth_and_dispersal_remix[2,22:41] 
    names(start_pop_sizes_remix) <- help_names2
    #only take 10 per mL as in the experiment
    start_pop_sizes_remix <- start_pop_sizes_remix /sum(start_pop_sizes_remix)*10
    
    start_pop_sizes_remix[which(is.na(start_pop_sizes_remix))] <- 0
    
    start_pop_sizes_remix[which(start_pop_sizes_remix < quasi_extinction_threshold)] <- 0
    
  }
  
  if(length(which.max(N_front[,max_it])) != 0){
    all_winners_front[act_glob_it] <- as.character(lines_data$clone[which.max(N_front[,max_it])])
  }else{
    all_winners_front[act_glob_it] <- rep("extinction",length(lines_data$clone))
  }
  
  if(length(which.max(N_core[,max_it])) != 0){
    all_winners_core[act_glob_it] <- as.character(lines_data$clone[which.max(N_core[,max_it])])
  }else{
    all_winners_core[act_glob_it] <- rep("extinction",length(lines_data$clone))
  }
  if(length(which.max(N_remix[,max_it])) != 0){
    all_winners_remix[act_glob_it] <- as.character(lines_data$clone[which.max(N_remix[,max_it])])
  }else{
    all_winners_remix[act_glob_it] <- rep("extinction",length(lines_data$clone))
  }
  
  # run time output
  print(act_glob_it)
  
}

#################################################
# plotting of winning probabilities
x11(width=13,height=5)
par(mfrow=c(1,3),mar=c(1,2,0,0),oma=c(5.5,4,4,0.5))
barplot(sort(table(all_winners_front),decreasing = T)/length(all_winners_front),las=3)
barplot(sort(table(all_winners_core),decreasing = T)/length(all_winners_core),las=3)
barplot(sort(table(all_winners_remix),decreasing = T)/length(all_winners_remix),las=3)
mtext(side=2,at=0.5,outer=T,"Probability of winning", line=2)
mtext(side=3,at=1/6,outer=T,"Front")
mtext(side=3,at=3/6,outer=T,"Core")
mtext(side=3,at=5/6,outer=T,"Control")

#################################################